/* 	This eden file illustrates the concept of automating a pattern
	of observation. The underlying construal is:
	The watch automatically consults the real-world time (via
	the action new_time = gettime(); in chk_time), and updates
	the number of seconds elapsed since 1/1/70 (recorded in tn)
	accordingly.
	Issues:
		How fast does chk_time act and schedule actions (todo())?
		"Accordingly" means "if the time recorded in new_time changes"?
		How much real-time elapses between tn = gettime(); and the
		initiation of the clock mechanism with first invocation of
		inc_time();?
*/

tn = gettime();

/* use a day long absolute time for testing purposes */
func abs_time {
	auto h,m,s;
	h=$1[3]; m=$1[2]; s=$1[1];
	return h*3600+m*60+s;
}

func ext_time {
      auto h,m,s;
	h = $1/3600;
	m = ($1%3600)/60;
	s = $1%60;
	return [s,m,h];
}

/* inc_time updates the time second by second */

proc inc_time {
	auto tnt, tailtn;
	tailtn = [tn[4],tn[5],tn[6],tn[7]];
	tnt = abs_time(tn);
	for (i=1; i<=$1; i++) {
		tnt++; tn = ext_time(tnt)//tailtn;
	}
}

old_time = gettime();
proc chk_time : new_time {
	todo("new_time = gettime();");
	if (old_time[1] != new_time[1])
	{
		inc_time(1);
		old_time = new_time;
	}
}

new_time = old_time;
/* need to initialise this in order to trigger chk_time */

/* the role of chk_time as controlling the update of tn through invocations of inc_time(1)is made more apparent if we contrast it with the following variant:

proc chk_time: tn {
	 todo("inc_time(1);");
}

which effectively updates the clock as fast as the eden queuing and processing speed permits
*/ 

tnsecs is abs_time(tn);